angular.module('starter.controllers', [])
  .controller('AppCtrl', function ($scope, $ionicPlatform, NSD, UUID, $ionicModal) {
  $scope.name = "MyName";
  $scope.connection = null;
  $scope.have_stream = false;
  $scope.webrtc = null;
  $scope.readyToCall = false;
  $scope.chatroom = null;
  $scope.localVideoSrc = "";
  $scope.popupText = "Loading...";
  $scope.videoScaling = "1";
  $scope.vid1 = document.getElementById("vid1");
  $scope.vid1Embedded = document.getElementById("vid1Embedded");
  $scope.vid2Embedded = document.getElementById("vid2Embedded");

  $scope.remoteVideoSrc = "";
  $scope.embeddedVideoVisible = false;  


  $ionicModal.fromTemplateUrl('partials/help.html', {
    scope: $scope,
    animation: 'none',
    backdropClickToClose: false,
    hardwareBackButtonClose: false,
  }).then(function(modal) {
    $scope.helpModal = modal;
    $scope.helpModal.show();
    $scope.helpModal.animation = 'slide-in-up';
  });

  $ionicModal.fromTemplateUrl('partials/wifi.html', {
    scope: $scope,
    backdropClickToClose: false,
    hardwareBackButtonClose: false,
  }).then(function(modal) {
    $scope.wifiModal = modal;
    $scope.wifiModal.animation = 'slide-in-up';
  });


  $scope.pairWithHost = function(host) {
    $scope.setPopupText("Pairing");
    console.log("Pairing with:");
    console.log(host);

    $scope.chatroom = host.description;
    console.log("Joining:" + $scope.chatroom);
    $scope.tryToJoin();

  }

  $scope.registerAsHost = function() {
    $scope.setPopupText("Awaiting peer");
    $scope.chatroom = UUID.generate();
    console.log("Starting:" + $scope.chatroom);

    $scope.tryToJoin();

    NSD.register($scope.name, "host", $scope.chatroom);
    //   showLoadingScreen();
  }

  $scope.tryToJoin = function() {

    console.log("Trying to join", $scope.readyToCall, $scope.chatroom);
    if ($scope.readyToCall && $scope.chatroom != null) {
      console.log("trying to connect");
      $scope.webrtc.joinRoom($scope.chatroom);
    }
  }

  $scope.setPopupText = function(text) {
    $scope.$apply(function() {$scope.popupText = text });
  };

  $scope.connect =function () {
    $scope.setPopupText("Finding friends");
    NSD.listHosts(function(hosts) {
      console.log("hosts");
      console.log(hosts);

      if (hosts.service.length > 0) {
        $scope.pairWithHost(hosts.service[0]);
      } else {
        $scope.registerAsHost();
      }
    });
  }

  $scope.displayPassthroughVideo = function() {
    document.getElementById("vid1").src = $scope.localVideoSrc;
    document.getElementById("vid2").src = $scope.localVideoSrc;
  };

  $scope.displayRemoteVideo = function() {
    document.getElementById("vid1").src = $scope.remoteVideoSrc;
    document.getElementById("vid2").src = $scope.remoteVideoSrc;
  };

  $scope.startAnalytics = function() {
    window.analytics.startTrackerWithId('UA-60137745-2');
    if (localStorage['user'] === undefined) {
      localStorage['user'] = UUID.generate();
    }

    window.analytics.setUserId(localStorage['user']);
    window.analytics.trackView('Help');
  }

  $scope.closeHelp = function() {
    $scope.helpModal.remove();
    window.analytics.trackView('Main');
  }

  $scope.pageClicked = function(event) {

    if (event.toElement.tagName === "BUTTON") {
      return;
    }

    $scope.magnetTriggered();  
  }

  $scope.magnetTriggered = function () {
    if ($scope.remoteVideoSrc === "")  {
      // Not connected to anyone - nothing to do!
      return;
    }

    if ($scope.embeddedVideoVisible) {
      $scope.displayPassthroughVideo();
      $scope.vid1Embedded.src = "";
      $scope.vid2Embedded.src = "";
      $scope.embeddedVideoVisible = false;
      return;
    }

    if ($scope.vid1.src === $scope.remoteVideoSrc) {
      $scope.vid1Embedded.src = $scope.localVideoSrc;
      $scope.vid2Embedded.src = $scope.localVideoSrc;
      $scope.embeddedVideoVisible = true;
      return;
    }

    if ($scope.vid1.src === $scope.localVideoSrc) {
      $scope.displayRemoteVideo();
      return;
    }

  }

  $scope.loadStubs = function() {
    var mobileOnly = function() { console.log("Sorry, but this is only available in cordova."); return true;};

    if (typeof ZeroConf === "undefined") {
      ZeroConf = {};

      ZeroConf.list = function(type, port, callback) {callback({service:[]});};
      ZeroConf.register = mobileOnly;
    }

    if (typeof window.analytics === "undefined") {
      $scope.startAnalytics = function(){};
      window.analytics = {};
      window.analytics.trackView = function() {};
      window.analytics.trackException = function() {};
    }

    if (typeof navigator.magnetsensor === "undefined"){
      navigator.magnetsensor = {};
      navigator.magnetsensor.onCardboardTriggerListener = function(){};
    }

    if (typeof navigator.connection === "undefined") {
      navigator.connection = {};
      navigator.connection.type = "wifi";
    }
  }

  $scope.initErrorCatcher = function() {
    var gOldOnError = window.onerror;
    // Override previous handler.
    window.onerror = function myErrorHandler(errorMsg, url, lineNumber) {

      window.analytics.trackException(errorMsg, Fatal);

      if (gOldOnError)
        // Call previous handler.
        return gOldOnError(errorMsg, url, lineNumber);

      // Just let default handler run.
      return false;
    }

  }
  
  $ionicPlatform.ready(function() {

    $scope.loadStubs();
    $scope.startAnalytics();
    $scope.initErrorCatcher();
    
    navigator.magnetsensor.onCardboardTriggerListener($scope.magnetTriggered, function(e) { console.log("Magnet failed:", e); })

    if (navigator.connection.type != "wifi") {
      window.analytics.trackView('Wifi')
      $scope.wifiModal.show();
    } else {
      $scope.init()
    }

    document.addEventListener("offline", function() { 
      window.analytics.trackView('Wifi');
      $scope.wifiModal.show();
    }, false);
    document.addEventListener("online", function(){ 
      if (navigator.connection.type != "wifi") {
        return; 
      } 
      $scope.wifiModal.hide();
      $scope.init()
    }, false);
  });

  $scope.init = function() {

    //    var height = 180;
    //    var width = 320;
    //    $scope.videoScaling = 1.15;

    //    var height = 720;
    //    var width = 1280;
    //    $scope.videoScaling = 1.5;

    var height = 480;
    var width = 640;
    $scope.videoScaling = 1.15;

    MediaStreamTrack.getSources(function(sources) { 
      var sourceId = null;
      sources.forEach(function(e) {
        if (e.facing === "environment"){
          sourceId = e.id;
        }
      })

      $scope.webrtc = new SimpleWebRTC({
        localVideoEl: 'vid1',
        autoRequestMedia: true,
        debug: false,
        media: {
          video: {optional:[{sourceId: sourceId}], mandatory: {
            minWidth: width,
            minHeight: height
          }},
          audio: false
        },
        detectSpeakingEvents: false,
        autoAdjustMic: false,
        //     ur: 'stun.l.google.com:19302'
      }); 

      $scope.webrtc.on('readyToCall', function () {
        console.log("Ready to call!");
        $scope.localVideoSrc = $scope.vid1.src;
        $scope.displayPassthroughVideo();
        $scope.readyToCall = true;
        $scope.tryToJoin();
      });

      $scope.webrtc.mute();

      $scope.webrtc.on('videoAdded', function (video, peer) {
        $scope.setPopupText("");

        window.analytics.trackView('Connected');
        console.log('video added', peer);

        $scope.remoteVideoSrc = video.src;
        $scope.displayRemoteVideo();

        NSD.unregister();

      });

      $scope.webrtc.on('videoRemoved', function (video, peer) {
        console.log("Video removed!");
        $scope.remoteVideoSrc = "";

        $scope.setPopupText("Connection lost");
        $scope.chatroom = null;
        $scope.webrtc.leaveRoom() 
        $scope.displayPassthroughVideo();
        $scope.connect();
      });

      $scope.connect();
    }); 

  };
});